function out = Monopoly
out{1} = @init;
out{2} = @fun_eval;
out{3} = @jacobian;
out{4} = @jacobianp;
out{5} = @hessians;
out{6} = @hessiansp;
out{7} = @der3;
out{8} = [];
out{9} = [];

% --------------------------------------------------------------------------
function dydt = fun_eval(t,kmrgd,delta)
P=3.6-2.4*(kmrgd(1)+kmrgd(2)) + 0.6*(kmrgd(1)*kmrgd(1) + kmrgd(1)*kmrgd(2) + kmrgd(2)*kmrgd(2)) - 0.05*(kmrgd(1)*kmrgd(1)*kmrgd(1) + kmrgd(1)*kmrgd(1)*kmrgd(2) + kmrgd(1)*kmrgd(2)*kmrgd(2) + kmrgd(2)*kmrgd(2)*kmrgd(2));
dydt=[kmrgd(2);
kmrgd(2)+delta*P;];

% --------------------------------------------------------------------------
function [tspan,y0,options] = init
handles = feval(Monopoly);
y0=[0,0];
options = odeset('Jacobian',handles(3),'JacobianP',handles(4),'Hessians',handles(5),'HessiansP',handles(6));
tspan = [0 10];

% --------------------------------------------------------------------------
function jac = jacobian(t,kmrgd,delta)
jac=[ 0 , 1 ; -delta*((kmrgd(1)*kmrgd(2))/10 - (3*kmrgd(2))/5 - (6*kmrgd(1))/5 + (3*kmrgd(1)^2)/20 + kmrgd(2)^2/20 + 12/5) , 1 - delta*((kmrgd(1)*kmrgd(2))/10 - (6*kmrgd(2))/5 - (3*kmrgd(1))/5 + kmrgd(1)^2/20 + (3*kmrgd(2)^2)/20 + 12/5) ];
% --------------------------------------------------------------------------
function jacp = jacobianp(t,kmrgd,delta)
jacp=[ 0 ; (3*kmrgd(1)*kmrgd(2))/5 - (12*kmrgd(2))/5 - (12*kmrgd(1))/5 - (kmrgd(1)*kmrgd(2)^2)/20 - (kmrgd(1)^2*kmrgd(2))/20 + (3*kmrgd(1)^2)/5 - kmrgd(1)^3/20 + (3*kmrgd(2)^2)/5 - kmrgd(2)^3/20 + 18/5 ];
% --------------------------------------------------------------------------
function hess = hessians(t,kmrgd,delta)
hess1=[ 0 , 0 ; -delta*((3*kmrgd(1))/10 + kmrgd(2)/10 - 6/5) , -delta*(kmrgd(1)/10 + kmrgd(2)/10 - 3/5) ];
hess2=[ 0 , 0 ; -delta*(kmrgd(1)/10 + kmrgd(2)/10 - 3/5) , -delta*(kmrgd(1)/10 + (3*kmrgd(2))/10 - 6/5) ];
hess(:,:,1) =hess1;
hess(:,:,2) =hess2;
% --------------------------------------------------------------------------
function hessp = hessiansp(t,kmrgd,delta)
hessp1=[ 0 , 0 ; (6*kmrgd(1))/5 + (3*kmrgd(2))/5 - (kmrgd(1)*kmrgd(2))/10 - (3*kmrgd(1)^2)/20 - kmrgd(2)^2/20 - 12/5 , (3*kmrgd(1))/5 + (6*kmrgd(2))/5 - (kmrgd(1)*kmrgd(2))/10 - kmrgd(1)^2/20 - (3*kmrgd(2)^2)/20 - 12/5 ];
hessp(:,:,1) =hessp1;
%---------------------------------------------------------------------------
function tens3  = der3(t,kmrgd,delta)
tens31=[ 0 , 0 ; -(3*delta)/10 , -delta/10 ];
tens32=[ 0 , 0 ; -delta/10 , -delta/10 ];
tens33=[ 0 , 0 ; -delta/10 , -delta/10 ];
tens34=[ 0 , 0 ; -delta/10 , -(3*delta)/10 ];
tens3(:,:,1,1) =tens31;
tens3(:,:,1,2) =tens32;
tens3(:,:,2,1) =tens33;
tens3(:,:,2,2) =tens34;
%---------------------------------------------------------------------------
function tens4  = der4(t,kmrgd,delta)
%---------------------------------------------------------------------------
function tens5  = der5(t,kmrgd,delta)
